/*****************************************************************************
 *
 *   Copyright(C) 2011, Embedded Artists AB
 *   All rights reserved.
 *
 ******************************************************************************/



/******************************************************************************
 * Includes
 *****************************************************************************/
#include "type.h"
#include "capsense.h"
#include "adc.h"
#include "comp.h"
#include "timer16.h"
#include "gpio.h"

/******************************************************************************
 * Defines and typedefs
 *****************************************************************************/

#define TOTAL_PLATES (4)
#define AVG_VALUES   (200)

/******************************************************************************
 * External global variables
 *****************************************************************************/



/******************************************************************************
 * Local variables
 *****************************************************************************/

__IO uint32_t * platePin[TOTAL_PLATES] = {	&(LPC_IOCON->PIO0_19),
											&(LPC_IOCON->PIO0_20),
											&(LPC_IOCON->PIO0_21),
											&(LPC_IOCON->PIO0_22)};

static uint8_t comparatorInput[TOTAL_PLATES] = {COMP0_INCHANNEL_P0IN0,
											COMP0_INCHANNEL_P0IN1,
											COMP0_INCHANNEL_P0IN2,
											COMP0_INCHANNEL_P0IN3};

static uint32_t average[TOTAL_PLATES];
static uint8_t debounce[TOTAL_PLATES];

static volatile uint8_t currentPlate = 0;

static uint8_t state[4];


/******************************************************************************
 * Local Functions
 *****************************************************************************/


void setPinAsOutput(__IO uint32_t * pin){

	*pin = 					(0x0<<0)|		// FUNC
			(0<<4)|			// Pull Up
			(0<<6)|			// Inverted
			(1<<7)|			// Analog/Digital
			(1<<9)|			// High/Low Current
			(0<<10)|		// Open-Drain
			(0x00<<11)|		// Sample mode input filter
			(0x00<<13);		// CLK_DIV for filter

}


void setPinAsInput(__IO uint32_t * pin){

	*pin = 					(0x2<<0)|		// FUNC
			(0<<4)|			// Pull Up
			(0<<6)|			// Inverted
			(0<<7)|			// Analog/Digital
			(1<<9)|			// High/Low Current
			(0<<10)|		// Open-Drain
			(0x00<<11)|		// Sample mode input filter
			(0x00<<13);		// CLK_DIV for filter
}


/******************************************************************************
 * Public Functions
 *****************************************************************************/


/******************************************************************************
 *
 * Description:
 *    Initialize the Capacitive sensor Driver
 *
 *****************************************************************************/
void capsense_init (void)
{
	int i = 0;

	TIM16_InitTypeDef TIM16_InitStruct;
	TIM16_CAPTURETypeDef TIM16_CaptureConfigStruct;
	TIM16_MATCHTypeDef TIM16_MatchConfigStruct;

	for (i = 0; i < 4; i++) {
		average[i] = 0;
		state[i] = 0;
		debounce[i] = 0;
	}

	// ------------------------
	// Comparator Configuration
	// ------------------------

	COMP_Init();

    // Configure voltage ladder: use Vdd(3V3) as reference and generate output voltage=10*3.3/31=1.06V
    COMP_VolLadderConfig (COMP_VLADREF_VDD, 10);

    // Enable voltage ladder
    COMP_VolLadderCmd (ENABLE);

    // Configure the input channel for +/- port of comparator 0
    COMP0_InputConfig (COMP_INPORT_NEGATIVE, COMP0_INCHANNEL_VLD);
    COMP0_InputConfig (COMP_INPORT_POSITIVE, comparatorInput[currentPlate]);

    // Configure the output
    COMP_OutputConfig (COMP_INDEX_0, COMP_OUTPUT_SYNC);

    COMP_Cmd (COMP_INDEX_0, ENABLE);

	// -------------------
	// Timer configuration
	// -------------------

	// Prescaler options

	TIM16_InitStruct.PrescaleOption = TIM16_PRESCALE_TICKVAL;
	TIM16_InitStruct.PrescaleValue = 1;

	TIM16_Init(LPC_CT16B0, TIM16_TIMER_MODE, &TIM16_InitStruct);

	// Capture options

	// Capture channel 2, CT16B0_CAP2: Comparator 0, level output(high or low) as the input
	TIM16_CaptureConfigStruct.CaptureChannel = TIM16_CAPTURE_CHANNEL2;
	// Disable capture on CAPn.2 rising edge
	TIM16_CaptureConfigStruct.RisingEdge = DISABLE;
	// Enable capture on CAPn.2 falling edge
	TIM16_CaptureConfigStruct.FallingEdge = ENABLE;
	// Generate capture interrupt
	TIM16_CaptureConfigStruct.IntOnCapture = ENABLE;

	TIM16_ConfigCapture(LPC_CT16B0, &TIM16_CaptureConfigStruct);


	// Match option

	// Match channel 0, MR0
	TIM16_MatchConfigStruct.MatchChannel = TIM16_MATCH_CHANNEL0;
	// Enable interrupt when MR0 matches the value in TC register
	TIM16_MatchConfigStruct.IntOnMatch   = TRUE;
	// Enable reset on MR0: TIMER will reset if MR0 matches it
	TIM16_MatchConfigStruct.ResetOnMatch = TRUE;
	// No Stop on MR0 if MR0 matches it
	TIM16_MatchConfigStruct.StopOnMatch  = FALSE;
	// Toggle MR0 pin if MR0 matches it
	TIM16_MatchConfigStruct.ExtMatchOutputType =TIM16_EXTMATCH_NOTHING;
	// Set Match value, count value of 48000 (48000 * 1/24Mhz = 2mS --> 500 Hz)
	TIM16_MatchConfigStruct.MatchValue   = 48000;

    TIM16_ConfigMatch(LPC_CT16B0,&TIM16_MatchConfigStruct);

    // -----------------
    // PIN configuration
    // -----------------

	// Set Plate pins as output and High level
	GPIO_SetDir(LPC_GPIO0,19,1);
	GPIO_SetHighLevel(LPC_GPIO0,19,1);
	GPIO_SetDir(LPC_GPIO0,20,1);
	GPIO_SetHighLevel(LPC_GPIO0,20,1);
	GPIO_SetDir(LPC_GPIO0,21,1);
	GPIO_SetHighLevel(LPC_GPIO0,21,1);
	GPIO_SetDir(LPC_GPIO0,22,1);
	GPIO_SetHighLevel(LPC_GPIO0,22,1);

	// Set as output
	setPinAsOutput(platePin[0]);
	setPinAsOutput(platePin[1]);
	setPinAsOutput(platePin[2]);
	setPinAsOutput(platePin[3]);

	// Initial discharge
	setPinAsInput(platePin[currentPlate]);

	// Start timer
	TIM16_ResetCounter(LPC_CT16B0);
	TIM16_Cmd(LPC_CT16B0, ENABLE);

	// Enable IRQs

	// preemption = 1, sub-priority = 1
	NVIC_SetPriority(TIMER_16_0_IRQn, ((0x01<<3)|0x01));
	// Enable interrupt for timer
	NVIC_EnableIRQ(TIMER_16_0_IRQn);
}

/******************************************************************************
 *
 * Description:
 *    Get state of the capacitive sensors
 *
 * Returns:
 *   state of the capacitive sensors
 *
 *****************************************************************************/
uint8_t capsense_read(void)
{
	return ( state[3] << 3 | state[2] << 2 | state[1] << 1 | state[0] << 0);
}

void TIMER16_0_IRQHandler(void)
{
	uint32_t int_status_reg;
    int reading;
    static short i=0;
    static int readed = 0;

    // Check the cause of the IRQ
    int_status_reg = TIM16_GetIntStatusReg (LPC_CT16B0);

	//--------------------------------------------------------------------------
    // Capture IRQ
	//--------------------------------------------------------------------------
    if (int_status_reg & TIM16_INT_CR2)
    {
    	// Clear Int Pending Bit
    	TIM16_ClearIntPendingBit(LPC_CT16B0,TIM16_INT_CR2);

    	// Get the capture value
    	reading = TIM16_GetCaptureValue(LPC_CT16B0, TIM16_CAPTURE_CHANNEL2);

    	// For the first 200 times, the base average is calculated
    	if (i<200)
    	{
    		// Check for valid range
    		if ((reading>0) && (reading<1000)){
    			average[currentPlate] += reading;
    			i++;
    			readed = 1;
    		}
    	}
    	else
    	{
    		// For the time AVG_VALUES, the average is calculated
    		if (i==AVG_VALUES){
    			//average[currentPlate] = average[currentPlate] / AVG_VALUES;
    			average[0] = average[0] / (AVG_VALUES/TOTAL_PLATES);
    			average[1] = average[1] / (AVG_VALUES/TOTAL_PLATES);
    			average[2] = average[2] / (AVG_VALUES/TOTAL_PLATES);
    			average[3] = average[3] / (AVG_VALUES/TOTAL_PLATES);
    			i++;
    		}

    		// Check for out of bounds readings (remove glitches)
    		if ((reading>0) && (reading<(average[currentPlate]<<2))){

    			// above (average + 6.25% of average) -> press condition
    			if (reading > average[currentPlate] + (average[currentPlate] >> 4))
    			{
    				// debounce, 4 successive triggers for press
    				if (debounce[currentPlate] == 4){
    					// reached max, indicate pressed
    					state[currentPlate] = 1;
    				}else{
    					// still going toward max
    					debounce[currentPlate] ++;
    				}
    			}
    			// below (average + 3.125% of average) -> release condition
    			else if (reading < average[currentPlate] + (average[currentPlate] >> 5))
    			{
    				if (debounce[currentPlate] == 0){
    					// reached min, indicate release
    					state[currentPlate] = 0;
    				}else{
    					// going toward min
    					debounce[currentPlate] --;
    				}
    			}
    		}
    	}
    	readed = 1;
    }

	//--------------------------------------------------------------------------
    // Match IRQ
	//--------------------------------------------------------------------------
    if (int_status_reg & TIM16_INT_MR0)
    {
    	TIM16_ClearIntPendingBit(LPC_CT16B0, TIM16_INT_MR0);

    	//TODO: read error flag for testing an error on the last measured plate

    	// Switch to next plate
    	currentPlate = (currentPlate+1)%TOTAL_PLATES;

    	// Re charge the Capacitors
    	setPinAsOutput(platePin[currentPlate]);

    	// Reset readed flag, for error checking
    	readed = 0;

    	// Change Comparator input
    	COMP0_InputConfig (COMP_INPORT_POSITIVE, comparatorInput[currentPlate]);

    	// Start discharge
    	setPinAsInput(platePin[currentPlate]);
    }
}



