/*****************************************************************************
 *   pcf8576.c:  Driver for the PCF8576 LCD controller
 *
 *   Copyright(C) 2011, Embedded Artists AB
 *   All rights reserved.
 *
 ******************************************************************************/

/*
 * NOTE: I2C must have been initialized before calling any functions in this
 * file.
 */

/******************************************************************************
 * Includes
 *****************************************************************************/

#include "type.h"
#include "i2c.h"

/******************************************************************************
 * Defines and typedefs
 *****************************************************************************/

#define PCF8576_I2C_ADDR	(0x38)

//From schematic
// segment  A = 0x00, 0x10
// segment  B = 0x00, 0x20
// segment  C = 0x00, 0x40
// segment  D = 0x80, 0x00
// segment  E = 0x40, 0x00
// segment  F = 0x20, 0x00
// segment  G = 0x00, 0x02
// segment  H = 0x00, 0x01
// segment  I = 0x01, 0x00
// segment  J = 0x02, 0x00
// segment  K = 0x04, 0x00
// segment  L = 0x08, 0x00
// segment  M = 0x00, 0x08
// segment  N = 0x00, 0x04
// segment CA = 0x10, 0x00
// segment DP = 0x00, 0x80

// 14 segment lookup table.
const uint8_t lcdSeg14Table[][3] = {
    { (uint8_t)'0', 0xE2, 0x74 },
    { (uint8_t)'1', 0x02, 0x60 },
    { (uint8_t)'2', 0xC4, 0x32 },
    { (uint8_t)'3', 0x84, 0x72 },
    { (uint8_t)'4', 0x24, 0x62 },
    { (uint8_t)'5', 0xA8, 0x12 },
    { (uint8_t)'6', 0xE4, 0x52 },
    { (uint8_t)'7', 0x02, 0x18 },
    { (uint8_t)'8', 0xE4, 0x72 },
    { (uint8_t)'9', 0xA4, 0x72 },
    { (uint8_t)'A', 0x64, 0x72 },
    { (uint8_t)'B', 0x85, 0x78 },
    { (uint8_t)'C', 0xE0, 0x10 },
    { (uint8_t)'D', 0x81, 0x78 },
    { (uint8_t)'E', 0xE4, 0x12 },
    { (uint8_t)'F', 0x64, 0x12 },
    { (uint8_t)'G', 0xE4, 0x50 },
    { (uint8_t)'H', 0x64, 0x62 },
    { (uint8_t)'I', 0x81, 0x18 },
    { (uint8_t)'J', 0xC0, 0x60 },
    { (uint8_t)'K', 0x6A, 0x02 },
    { (uint8_t)'L', 0xE0, 0x00 },
    { (uint8_t)'M', 0x62, 0x61 },
    { (uint8_t)'N', 0x68, 0x61 },
    { (uint8_t)'O', 0xE0, 0x70 },
    { (uint8_t)'P', 0x64, 0x32 },
    { (uint8_t)'Q', 0xE8, 0x70 },
    { (uint8_t)'R', 0x6C, 0x32 },
    { (uint8_t)'S', 0x84, 0x51 },
    { (uint8_t)'T', 0x01, 0x18 },
    { (uint8_t)'U', 0xE0, 0x60 },
    { (uint8_t)'V', 0x62, 0x04 },
    { (uint8_t)'W', 0x68, 0x64 },
    { (uint8_t)'X', 0x0A, 0x05 },
    { (uint8_t)'Y', 0x02, 0x09 },
    { (uint8_t)'Z', 0x82, 0x14 },
    { (uint8_t)' ', 0x00, 0x00 },
    { (uint8_t)'/', 0x02, 0x04 },
    { (uint8_t)'*', 0x0B, 0x0D },
    { (uint8_t)'+', 0x05, 0x0A },
    { (uint8_t)'-', 0x04, 0x02 },
    { (uint8_t)'\'', 0x01, 0x00 },
    { (uint8_t)'\"', 0x01, 0x20 },
    { (uint8_t)'(', 0x0A, 0x00 },
    { (uint8_t)')', 0x00, 0x05 },
    { (uint8_t)',', 0x00, 0x04 },
    { (uint8_t)'_', 0x80, 0x00 },
    { (uint8_t)':', 0x84, 0x02 },
    { (uint8_t)'=', 0x84, 0x02 },
    { (uint8_t)'o', 0x24, 0x32 },  //degrees
    { (uint8_t)'#', 0xFF, 0xFF },
};

/******************************************************************************
 * External global variables
 *****************************************************************************/

/******************************************************************************
 * Local variables
 *****************************************************************************/
static uint8_t lcdFrame[20];
static uint16_t lcdInitialMsDelay;
static uint16_t lcdScrollSpeedMsPerChar;
static uint16_t lcdScrollElapsedTime;
static uint8_t lcdStringOffset;
static uint8_t *pLcdString;
static uint8_t lcdScrollActive = FALSE;

/******************************************************************************
 * Local Functions
 *****************************************************************************/
#define I2C_CODE_OK    1
#define I2C_CODE_ERR  50

static int8_t
pcf8576(uint8_t lmAddr, uint8_t* pBuf, uint16_t len, uint8_t* pBuf2, uint16_t len2)
{
	int8_t retCode = I2C_CODE_OK;


  // write only
  if (len2 == 0) {
	  I2CWrite((lmAddr << 1), pBuf, len);
  }
  else {
	  I2CWrite((lmAddr << 1), pBuf, len);
	  I2CRead((lmAddr << 1), pBuf2, len2);
  }

  return retCode;
}

/******************************************************************************
 * Public Functions
 *****************************************************************************/

/******************************************************************************
 *
 * Description:
 *    Initialize the EEPROM Driver
 *
 *****************************************************************************/
void pcf8576_init (void)
{
    lcdFrame[0] = 0x4C|0x80;  // Mode set (1/3 bias, enabled, static display), more commands
    lcdFrame[1] = 0x60|0x80;  // Select device 0, more commands
    lcdFrame[2] = 0x70|0x80;  // No blinking, more commands
    lcdFrame[3] = 0x00;		  // Reset data pointer, last command

    lcdFrame[4] = 0;      //Char#1 part1
    lcdFrame[5] = 0;      //Char#2 part1
    lcdFrame[6] = 0;      //Char#3 part1
    lcdFrame[7] = 0;      //Char#4 part1
    lcdFrame[8] = 0;      //Char#5 part1
    lcdFrame[9] = 0;      //Char#6 part1
    lcdFrame[10]= 0;      //Char#7 part1
    lcdFrame[11]= 0;      //Char#8 part1
//
    lcdFrame[12]= 0;      //Char#8 part2
    lcdFrame[13]= 0;      //Char#7 part2
    lcdFrame[14]= 0;      //Char#6 part2
    lcdFrame[15]= 0;      //Char#5 part2
    lcdFrame[16]= 0;      //Char#4 part2
    lcdFrame[17]= 0;      //Char#3 part2
    lcdFrame[18]= 0;      //Char#2 part2
    lcdFrame[19]= 0;      //Char#1 part2

    //pcf8576(i2c-addr,            *pBuf,   len, *pBuf2, len2
    pcf8576(PCF8576_I2C_ADDR, &lcdFrame[0],  20,   NULL,   0);
}


void lcdWriteDigit(uint8_t character, uint8_t position)
{
    uint8_t tableIndex;

    // Search the table for a match
    // If none is found, use the last entry
    for (tableIndex=0; tableIndex<49; tableIndex++)
    {
        if (lcdSeg14Table[tableIndex][0] == character)
        {
            break;
        }
    }
    lcdFrame[position + 4]  = lcdSeg14Table[tableIndex][1];
    lcdFrame[19 - position] = lcdSeg14Table[tableIndex][2];
}

void lcdAddDpAfterDigit(uint8_t position)
{
    lcdFrame[19 - position] |= 0x80;
}

void lcdFlushDigits(void)
{
//  pcf8576(i2c-addr,         *pBuf,        len, *pBuf2, len2
	pcf8576(PCF8576_I2C_ADDR, &lcdFrame[0],  20,   NULL,   0);
}

void lcdTimerHandler(uint16_t msPassed)
{
	uint8_t i;

	//check if scrolling is active
	if (lcdScrollActive == TRUE)
	{
		//check if initial timeout is still active
		if (lcdInitialMsDelay > 0)
		{
			if (lcdInitialMsDelay > msPassed)
				lcdInitialMsDelay -= msPassed;
			else
				lcdInitialMsDelay = 0;
		}

		//check if time to update
		if (lcdInitialMsDelay == 0)
		{
			lcdScrollElapsedTime += msPassed;

			if (lcdScrollElapsedTime >= lcdScrollSpeedMsPerChar)
			{
				lcdScrollElapsedTime -= lcdScrollSpeedMsPerChar;

				//update string offset and check if end of string reached
				lcdStringOffset++;
				if ((pLcdString[lcdStringOffset+8]) == 0)
					lcdScrollActive = FALSE;
				//copy first 8 characters
				for (i=0; i<8; i++)
					lcdWriteDigit(pLcdString[lcdStringOffset+i], i);
				lcdFlushDigits();
			}
		}
	}
}

void lcdWriteString(uint8_t *pString, uint16_t initialMsDelay, uint16_t scrollSpeedMsPerChar)
{
	uint8_t i;

	lcdScrollActive = FALSE;

	lcdInitialMsDelay = initialMsDelay;
	lcdScrollSpeedMsPerChar = scrollSpeedMsPerChar;
	lcdScrollElapsedTime = 0;
	lcdStringOffset = 0;
	pLcdString = pString;

	//copy first 8 characters
	for (i=0; i<8; i++)
		lcdWriteDigit(pString[i], i);
	lcdFlushDigits();
	lcdScrollActive = TRUE;
}

uint8_t lcdIsScrollActive(void)
{
	return lcdScrollActive;
}

void lcdDisableScrolling(void)
{
	lcdScrollActive = FALSE;
}
