/*****************************************************************************
 *
 *   Copyright(C) 2011, Embedded Artists AB
 *   All rights reserved.
 *
******************************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/

#include "LPC122x.h"
#include "i2c.h"
#include "adc.h"
#include "lm75b.h"
#include "pcf8576.h"
#include "comp.h"
#include "eeprom.h"
#include "uart.h"
#include "capsense.h"

#include <cr_section_macros.h>
#include <NXP/crp.h>

/******************************************************************************
 * Typedefs and defines
 *****************************************************************************/

#define TIME_INTERVAL (500)

#define BTN_SW1 (0x01)
#define BTN_SW2 (0x02)
#define BTN_SW4 (0x04)

#define LED1 (0x01)
#define LED2 (0x02)
#define LED3 (0x04)

#define STATE_SHOW_TRIM       (1)
#define STATE_SHOW_BUTTONS    (2)
#define STATE_SHOW_TEMP       (3)
#define STATE_SHOW_EEPROM_RTC (4)


/******************************************************************************
 * Global variables
 *****************************************************************************/

// Variable to store CRP value in. Will be placed automatically
// by the linker when "Enable Code Read Protect" selected.
// See crp.h header for more information
__CRP const unsigned int CRP_WORD = CRP_NO_CRP;

volatile uint32_t msTicks=0;

/******************************************************************************
 * Local variables
 *****************************************************************************/


static uint8_t state = 0;

/******************************************************************************
 * Local functions
 *****************************************************************************/

static void rtc_init(void)
{
	LPC_SYSCON->SYSAHBCLKCTRL |= (1<<19);

	LPC_RTC->LR = 0;

	// enable RTC
	LPC_RTC->CR = 0x01;
}

static void buttons_init(void)
{
	// SW1 -> GPIO, input
	LPC_IOCON->PIO0_5 &= ~0x07;
	LPC_GPIO0->DIR &= ~(1 << 5);

	// SW2 -> GPIO, input
	LPC_IOCON->PIO1_3 &= ~0x07;
	LPC_GPIO1->DIR &= ~(1 << 3);

	// SW4 -> GPIO, input
	LPC_IOCON->PIO0_12 &= ~0x07;
	LPC_GPIO0->DIR &= ~(1 << 12);
}

static uint8_t buttons_getState(void)
{
	uint8_t s = 0;
	uint32_t p0 = LPC_GPIO0->PIN;
	uint32_t p1 = LPC_GPIO1->PIN;

	if ((p0 & (1<<5)) == 0) {
		s |= BTN_SW1;
	}

	if ((p0 & (1<<12)) == 0) {
		s |= BTN_SW4;
	}

	if ((p1 & (1<<3)) == 0) {
		s |= BTN_SW2;
	}

	return s;
}

static void leds_init(void)
{
	// LED1 connected to p0.12 which is controlled through SW4

	// LED2 -> GPIO, output, off
	LPC_IOCON->PIO0_27 &= ~0x07;
	LPC_GPIO0->DIR |= 1 << 27;
	LPC_GPIO0->CLR  = 1 << 27;

	// LED3 -> GPIO, output, off
	LPC_IOCON->PIO0_28 &= ~0x07;
	LPC_GPIO0->DIR |= 1 << 28;
	LPC_GPIO0->CLR  = 1 << 28;

	// LED4 pwr
	// LED5 UART RX
	// LED6 UART TX
	// LED7 Reset
}

static void leds_setState(uint8_t ledOnMask)
{
	if ((ledOnMask & LED2) != 0) {
		LPC_GPIO0->SET  = 1 << 27;
	}
	else {
		LPC_GPIO0->CLR  = 1 << 27;
	}

	if ((ledOnMask & LED3) != 0) {
		LPC_GPIO0->SET  = 1 << 28;
	}
	else {
		LPC_GPIO0->CLR  = 1 << 28;
	}
}



/******************************************************************************
 * Public functions
 *****************************************************************************/

int main(void)
{
	uint32_t time = 0;
	uint32_t rtcTime1=0;
	uint32_t rtcTime2=0;
	uint32_t rtc1 = 0;
	uint32_t rtc2 = 0;
	uint8_t eepromOk = 0;
	I2C_InitTypeDef i2cInit;
	UART_CFG_Type uartCfg;
	uint8_t btns = 0;
	uint32_t adcVal = 0;
	int32_t tempValue;
	uint8_t showRtc = 0;
	uint8_t cap = 0;


	SysTick_Config(SystemCoreClock / 1000);
	NVIC_SetPriority(SysTick_IRQn, 2);

	// initialize I2C
    i2cInit.ClockRate = 100000;
    i2cInit.Mode      = I2C_MASTER;
    i2cInit.InterruptMode = I2C_INTERRUPT_MODE;
    I2C_Init(&i2cInit);
	NVIC_SetPriority(I2C_IRQn, 1);

	/* GPIO */
	LPC_SYSCON->SYSAHBCLKCTRL |= 0xE0010001;

	// initialize UART connected to USB-to-UART bridge

	LPC_SYSCON->SYSAHBCLKCTRL |= (1<<12);
	LPC_SYSCON->UART0CLKDIV = 1;
	LPC_IOCON->PIO0_1 &= ~0x07;
	LPC_IOCON->PIO0_1 |= 0x02;
	LPC_IOCON->PIO0_2 &= ~0x07;
	LPC_IOCON->PIO0_2 |= 0x02;

	UART_Init(LPC_UART0);
	uartCfg.baudrate = 115200;
	uartCfg.databits = UART_CFG_DATABIT_8;
	uartCfg.stopbits = UART_CFG_STOPBIT_1;
	uartCfg.parity   = UART_CFG_PARITY_NONE;
	uartCfg.fifodma  = UART_CFG_DMAMODE_DISABLE;
	uartCfg.fifolevel= UART_CFG_FIFOTRG_1;
	uartCfg.txdbreak = UART_CFG_TXDBREAK_DISABLE;

	UART_SetConfig(LPC_UART0, &uartCfg);
	UART_ConfigTXD(LPC_UART0, ENABLE);

	ADCInit(9000000);

	capsense_init();

	buttons_init();
	leds_init();

	// initialize RTC
	rtc_init();

	rtcTime1 = msTicks;

	UART_SendString(LPC_UART0, (uint8_t*)"LPC12D27 QSB Demo\r\n");

	// initialize LCD controller
	pcf8576_init();
	lcdWriteString((uint8_t*)"        THANK YOU FOR USING THE LPC12D27 QUICKSTART BOARD FROM EMBEDDED ARTISTS - BASED ON NXP'S AMAZING CORTEX-M0 LPC12D27        ", 1500, 350);

	// initialize the temperature sensor
	lm75b_config(LM75B_CFG_NORMAL);

	// check if the eeprom is OK
	eepromOk = eeprom_test();

	while (1) {

		// take a snapshot of RTC value when app has run at least 1 s
		if (rtc1 == 0 && msTicks > rtcTime1 + 1000) {
			rtc1 = LPC_RTC->DR;
			rtcTime2 = msTicks;
		}

		if (rtc2 == 0 && msTicks > rtcTime2 + 2100) {
			rtc2 = LPC_RTC->DR;
		}

		cap = capsense_read();
		if (cap & CAP_0) {
			state = STATE_SHOW_TRIM;
		}
		else if (cap & CAP_1) {
			state = STATE_SHOW_BUTTONS;
		}
		else if (cap & CAP_2) {
			state = STATE_SHOW_TEMP;
		}
		else if (cap & CAP_3) {
			state = STATE_SHOW_EEPROM_RTC;
		}


		switch(state) {
		case STATE_SHOW_TRIM:

			if (msTicks < time + 100) {
				break;
			}
			time = msTicks;

			lcdDisableScrolling();

			adcVal = ADCRead(6);

			lcdWriteDigit('A', 0);
			lcdWriteDigit('D', 1);
			lcdWriteDigit('C', 2);
			lcdWriteDigit(':', 3);
			lcdWriteDigit('0' + (adcVal / 1000), 4);
			lcdWriteDigit('0' + ((adcVal / 100) % 10), 5);
			lcdWriteDigit('0' + ((adcVal / 10) % 10), 6);
			lcdWriteDigit('0' + (adcVal % 10), 7);
			lcdFlushDigits();

			break;
		case STATE_SHOW_BUTTONS:

			if (msTicks < time + 100) {
				break;
			}
			time = msTicks;

			lcdDisableScrolling();

			btns = buttons_getState();

			lcdWriteDigit('B', 0);
			lcdWriteDigit('T', 1);
			lcdWriteDigit('N', 2);
			lcdWriteDigit(':', 3);

			lcdWriteDigit(' ', 4);
			lcdWriteDigit(' ', 5);
			lcdWriteDigit(' ', 6);
			lcdWriteDigit(' ', 7);

			leds_setState(0);

			if ((btns & BTN_SW1) != 0) {
				lcdWriteDigit('S', 5);
				lcdWriteDigit('W', 6);
				lcdWriteDigit('1', 7);

				leds_setState(LED2);
			}


			if ((btns & BTN_SW2) != 0) {
				lcdWriteDigit('S', 5);
				lcdWriteDigit('W', 6);
				lcdWriteDigit('2', 7);

				leds_setState(LED3);
			}

			if ((btns & BTN_SW4) != 0) {
				lcdWriteDigit('S', 5);
				lcdWriteDigit('W', 6);
				lcdWriteDigit('4', 7);
			}

			lcdFlushDigits();

			break;
		case STATE_SHOW_TEMP:

			if (msTicks < time + 500) {
				break;
			}
			time = msTicks;


			tempValue = lm75b_readTemp() / 10;
			lcdDisableScrolling();

			lcdWriteDigit('T', 0);
			lcdWriteDigit(':', 1);
			lcdWriteDigit(' ', 2);
			lcdWriteDigit('0' + ((tempValue / 100) % 10), 3);
			lcdWriteDigit('0' + ((tempValue / 10) % 10), 4);
			lcdAddDpAfterDigit(4);
			lcdWriteDigit('0' + (tempValue % 10), 5);
			lcdWriteDigit('o', 6);
			lcdWriteDigit('C', 7);
			lcdFlushDigits();

			break;
		case STATE_SHOW_EEPROM_RTC:

			if (msTicks > time + 1000) {
				time = msTicks;
				showRtc = !showRtc;
			}

			if (!showRtc) {
				if (eepromOk) {
					lcdWriteString((uint8_t *)"E2P OK  ", 1500, 350);
				}
				else {
					lcdWriteString((uint8_t *)"E2P FAIL", 1500, 350);
				}
			}
			else {
				if (rtc2 - rtc1 >= 2) {
					lcdWriteString((uint8_t *)"RTC OK  ", 1500, 350);
				}
				else {
					lcdWriteString((uint8_t *)"RTC FAIL", 1500, 350);
				}
			}




			break;
		}

	}



}

void SysTick_Handler(void)
{
	msTicks++;
	lcdTimerHandler(1);
}


